@@ -1,12 +1,12 @@
 #' Composed error multivariate distribution object for mgcv
 #'
-#' The comperr_mv family implements the composed error multivariate distribution in which the \eqn{\mu_1}, \eqn{\sigma_{V1}}, \eqn{\sigma_{U1}}, (or \eqn{\lambda_1}), \eqn{\mu_2}, \eqn{\sigma_{V2}}, \eqn{\sigma_{U2}}, (or \eqn{\lambda_2}) and \eqn{\tau} can depend on additive predictors. Useable only with \code{mgcv::gam}, the additive predictors are specified via a list of formulae.
+#' The comperr_mv family implements the composed error multivariate distribution in which the \eqn{\mu_1}, \eqn{\sigma_{V1}}, \eqn{\sigma_{U1}}, (or \eqn{\lambda_1}), \eqn{\mu_2}, \eqn{\sigma_{V2}}, \eqn{\sigma_{U2}}, (or \eqn{\lambda_2}) and \eqn{\delta} can depend on additive predictors. Useable only with \code{\link[mgcv:gam]{gam()}}, the additive predictors are specified via a list of formulae.
 #'
 #'
-#' @param link seven item list specifying the link for the \eqn{\mu_1}, \eqn{\sigma_{V1}}, \eqn{\sigma_{U1}}, \eqn{\lambda_1}, \eqn{\mu_2}, \eqn{\sigma_{V2}}, \eqn{\sigma_{U2}}, \eqn{\lambda_2} and \eqn{\tau} parameters. See details.
+#' @param link seven item list specifying the link for the \eqn{\mu_1}, \eqn{\sigma_{V1}}, \eqn{\sigma_{U1}}, \eqn{\lambda_1}, \eqn{\mu_2}, \eqn{\sigma_{V2}}, \eqn{\sigma_{U2}}, \eqn{\lambda_2} and \eqn{\delta} parameters. See details.
 #' @param s1 \eqn{s_1=-1} for production and \eqn{s_1=1} for cost function for margin 1.
 #' @param s2 \eqn{s_2=-1} for production and \eqn{s_2=1} for cost function for margin 2.
-#' @param dist vector of length three, specifying the bivariate distribution.  First element is the name of the first marginal distribution. Second element is the name of the second marginal distribution. Third element specifies the copula. See \code{\link{dcop}} for more details.
+#' @param family_mv vector of length three, specifying the bivariate distribution.  First element is the name of the first marginal distribution. Second element is the name of the second marginal distribution. Third element specifies the copula. See \code{dcop()} for more details.
 #'
 #' @return An object inheriting from class \code{general.family} of the 'mgcv' package, which can be used in the dsfa package.
 #'
@@ -18,9 +18,9 @@
 #'   \item The fourth formula specifies the response of margin 2 on the left hand side and the structure of the additive predictor for \eqn{\mu_2} parameter on the right hand side. Link function is "identity".
 #'   \item The fifth formula is one sided, specifying the additive predictor for the  \eqn{\sigma_{V2}} on the right hand side. Link function is "log".
 #'   \item The sixth formula  is one sided, specifying the additive predictor for the  \eqn{\sigma_{U2}} or \eqn{\lambda_2} on the right hand side. Link function is "log".
-#'   \item The seventh formula  is one sided, specifying the additive predictor for the  \eqn{\tau} on the right hand side. Link function is "tanh".
+#'   \item The seventh formula  is one sided, specifying the additive predictor for the  \eqn{\delta} on the right hand side. Link function is "glogit".
 #' }
-#' The fitted values and linear predictors for this family will be seven column matrices. The first column is the \eqn{\mu_1}, the second column is the \eqn{\sigma_{V1}}, the third column is \eqn{\sigma_{U1}}...
+#' The fitted values and linear predictors for this family will be seven column matrices. The columns correspond with the order of the formulae for the parameters.
 #'
 #' @examples
 #' \donttest{
@@ -41,13 +41,13 @@
 #' mu2=3+2*x4 #cost function parameter 2
 #' sigma_v2=exp(-1.5+0.75*x5) #noise parameter 2
 #' sigma_u2=exp(-1+.75*x6) #inefficiency parameter 2
-#' Tau<-(exp(1+2.5*x7)-1)/(exp(1+2.5*x7)+1) #Kendall's tau
+#' delta<-(exp(1+2.5*x7)-1)/(exp(1+2.5*x7)+1) #delta
 #'
 #' #Simulate responses and create dataset
 #' Y<-rcomperr_mv(n=N,
-#'                mu1=mu1, sigma_v1=sigma_v1, par_u1 = sigma_u1, s1=s1, dist1="normhnorm",
-#'                mu2=mu2, sigma_v2=sigma_v2, par_u2 = sigma_u2, s2=s2, dist2="normhnorm",
-#'                Tau=Tau, family=1)
+#'                mu1=mu1, sigma_v1=sigma_v1, par_u1 = sigma_u1, s1=s1,
+#'                mu2=mu2, sigma_v2=sigma_v2, par_u2 = sigma_u2, s2=s2,
+#'                delta=delta, family=c("normhnorm","normhnorm","normal"))
 #' dat<-data.frame(y1=Y[,1],y2=Y[,2], x1, x2, x3, x4, x5, x6, x7)
 #'
 #' #Write formulae for parameters
@@ -57,13 +57,14 @@
 #' mu_2_formula<-y2~x4
 #' sigma_v2_formula<-~x5
 #' sigma_u2_formula<-~x6
-#' tau_formula<-~x7
+#' delta_formula<-~x7
 #'
 #' #Fit model
 #' model<-mgcv::gam(formula=list(mu_1_formula,sigma_v1_formula,sigma_u1_formula,
 #'                               mu_2_formula,sigma_v2_formula,sigma_u2_formula,
-#'                               tau_formula),
-#'                  data=dat, family=comperr_mv(s1=s1, s2=s2, dist=c("normhnorm","normhnorm",1)),
+#'                               delta_formula),
+#'                  data=dat,
+#'                  family=comperr_mv(s1=s1, s2=s2, family_mv=c("normhnorm","normhnorm","normal")),
 #'                  optimizer="efs")
 #'
 #' #Model summary
@@ -75,18 +76,49 @@
 #' \item \insertRef{wood2017generalized}{dsfa}
 #' }
 #' @export
-comperr_mv <- function(link = list("identity", "log", "log", "identity", "log", "log", "tanh"), s1 = -1, s2 = -1, dist=c("normhnorm","normhnorm",1)){
+comperr_mv <- function(link = list("identity", "log", "log", "identity", "log", "log", "glogit"), s1 = -1, s2 = -1, family_mv=c("normhnorm","normhnorm","normal")){
   #Object for mgcv::gam such that the composed error multivariate distribution can be estimated.
 
   #Number of parameters
   npar <- 7
 
+  #Delta parameter space boundaries
+  if(family_mv[3]=="independent"){
+    a<-0
+    b<-1
+  }
+
+  if(family_mv[3]=="normal"){
+    a<--1
+    b<-1
+  }
+
+  if(family_mv[3]=="clayton"){
+    a<-0+1e-16
+    b<-28
+  }
+
+  if(family_mv[3]=="gumbel"){
+    a<-1
+    b<-17
+  }
+
+  if(family_mv[3]=="frank"){
+    a<--35
+    b<-35
+  }
+
+  if(family_mv[3]=="joe"){
+    a<-1+1e-16
+    b<-30
+  }
+
   #Link functions
   if (length(link) != npar) stop("comperr_mv requires 7 links specified as character strings")
-  okLinks <- list("identity", "log", "log","identity", "log", "log", "tanh")
+  okLinks <- list("identity", "log", "log","identity", "log", "log", "glogit")
   stats <- list()
-  param.names <- c("mu1", "sigma_v1", "par_u1","mu2", "sigma_v2", "par_u2","Tau")
-  for (i in 1:6) { # Links for "mu1", "sigma_v1", "sigma_u1","mu2", "sigma_v2", "sigma_u2","Tau"
+  param.names <- c("mu1", "sigma_v1", "par_u1","mu2", "sigma_v2", "par_u2","delta")
+  for (i in 1:6) { # Links for "mu1", "sigma_v1", "sigma_u1","mu2", "sigma_v2", "sigma_u2","delta"
     if (link[[i]] %in% okLinks[[i]]) stats[[i]] <- stats::make.link(link[[i]]) else
       stop(link[[i]]," link not available for ", param.names[i]," parameter of comperr_mv")
     fam <- structure(list(link=link[[i]],canonical="none",linkfun=stats[[i]]$linkfun,
@@ -101,15 +133,13 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
     stats[[7]] <- list()
     stats[[7]]$valideta <- function(eta) TRUE
     stats[[7]]$link = link[[7]]
-    # y<-log((x+1)/(1-x))
-    # x<-(exp(y)-1)/(exp(y)+1)
 
-    stats[[7]]$linkfun <- eval(parse(text = paste("function(mu) log((mu+1)/(1-mu))")))
-    stats[[7]]$linkinv <- eval(parse(text = paste("function(eta) (exp(eta)-1)/(exp(eta)+1)")))
-    stats[[7]]$mu.eta <- eval(parse(text = paste("function(eta) 1/(1 + cosh(eta))")))
-    stats[[7]]$d2link <- eval(parse(text = paste("function(mu) 4*mu/(mu^2-1)^2"))) #-((2 * exp(eta) * (-1 + exp(eta)))/(1 + exp(eta))^3)
-    stats[[7]]$d3link <- eval(parse(text = paste("function(mu) -4*(3*mu^2+1)/(mu^2-1)^3"))) #(2 * exp(eta) * (1 - 4 * exp(eta) + exp(2 * eta)))/(1 + exp(eta))^4
-    stats[[7]]$d4link <- eval(parse(text = paste("function(mu) 48*(mu^3+mu)/(mu^2-1)^4"))) #-((2 * exp(eta)*  (-1 + 11 * exp(eta) - 11 * exp(2 * eta) + exp(3 * eta)))/(1 + exp(eta))^5)
+    stats[[7]]$linkfun <- eval(parse(text = paste("function(mu) log((-",a,"+mu)/(",b,"-mu))")))#eval(parse(text = paste("function(mu) log((mu+1)/(1-mu))")))
+    stats[[7]]$linkinv <- eval(parse(text = paste("function(eta) exp(eta)/(1+exp(eta))*(",b,"-",a,")+",a)))
+    stats[[7]]$mu.eta <- eval(parse(text = paste("function(eta) exp(eta)*(",b,"-",a,")/(exp(eta)+1)^2")))
+    stats[[7]]$d2link <- eval(parse(text = paste("function(mu) 1/(",b,"-mu)^2-1/(",a,"-mu)^2")))#eval(parse(text = paste("function(mu) 4*mu/(mu^2-1)^2"))) #-((2 * exp(eta) * (-1 + exp(eta)))/(1 + exp(eta))^3)
+    stats[[7]]$d3link <- eval(parse(text = paste("function(mu) 2*(1/(",b,"-mu)^3+1/(-",a,"+mu)^3)"))) #(2 * exp(eta) * (1 - 4 * exp(eta) + exp(2 * eta)))/(1 + exp(eta))^4
+    stats[[7]]$d4link <- eval(parse(text = paste("function(mu) 6/(",b,"-mu)^4-6/(",a,"-mu)^4"))) #-((2 * exp(eta)*  (-1 + 11 * exp(eta) - 11 * exp(2 * eta) + exp(3 * eta)))/(1 + exp(eta))^5)
   }
   else stop(link[[7]], " link not available comperr_mv distribution")
 
@@ -118,9 +148,9 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
 
     type <- match.arg(type)
     para1<-reparametrize(mu = object$fitted[,1], sigma_v = object$fitted[,2], par_u = object$fitted[,3],
-                         s = object$family$s1, dist = object$family$dist[1])
+                         s = object$family$s1, family = object$family$family_mv[1])
     para2<-reparametrize(mu = object$fitted[,4], sigma_v = object$fitted[,5], par_u = object$fitted[,6],
-                         s = object$family$s2, dist = object$family$dist[2])
+                         s = object$family$s2, family = object$family$family_mv[2])
 
 
     rsd <- object$y-cbind(para1$mean,para2$mean)
@@ -163,12 +193,12 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
     sigma_v2 <- family$linfo[[5]]$linkinv( eta4 )
     par_u2   <- family$linfo[[6]]$linkinv( eta5 )
     s2       <- family$s2
-    Tau      <- family$linfo[[7]]$linkinv( eta6 )
+    delta      <- matrix(family$linfo[[7]]$linkinv( eta6 ), nrow=n)
 
     ##Define parameters
-    l0<-dcomperr_mv(x1=y[,1], mu1=mu1, sigma_v1=sigma_v1, par_u1=par_u1, s1=s1, dist1 = family$dist[1],
-                    x2=y[,2], mu2=mu2, sigma_v2=sigma_v2, par_u2=par_u2, s2=s2, dist2 = family$dist[2],
-                     Tau=Tau, family=family$dist[3], deriv=2, xg=family$tri, log.p = TRUE)
+    l0<-dcomperr_mv(x1=y[,1], mu1=mu1, sigma_v1=sigma_v1, par_u1=par_u1, s1=s1,
+                    x2=y[,2], mu2=mu2, sigma_v2=sigma_v2, par_u2=par_u2, s2=s2,
+                    delta=delta, family_mv=family$family_mv, deriv=2, tri=family$tri, log.p = TRUE)
 
     l<-sum(l0)
 
@@ -189,7 +219,7 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
                   family$linfo[[4]]$d2link(mu2),
                   family$linfo[[5]]$d2link(sigma_v2),
                   family$linfo[[6]]$d2link(par_u2),
-                  family$linfo[[7]]$d2link(Tau))
+                  family$linfo[[7]]$d2link(delta))
     }
 
     l1<-attr(l0,"gradient")
@@ -205,7 +235,7 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
                   family$linfo[[4]]$d3link(mu2),
                   family$linfo[[5]]$d3link(sigma_v2),
                   family$linfo[[6]]$d3link(par_u2),
-                  family$linfo[[7]]$d3link(Tau))
+                  family$linfo[[7]]$d3link(delta))
 
       l3<-attr(l0,"l3")
 
@@ -218,7 +248,7 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
                   family$linfo[[4]]$d4link(mu2),
                   family$linfo[[5]]$d4link(sigma_v2),
                   family$linfo[[6]]$d4link(par_u2),
-                  family$linfo[[7]]$d4link(Tau))
+                  family$linfo[[7]]$d4link(delta))
 
       l4<-attr(l0,"l4")
     }
@@ -251,60 +281,60 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
       start <- rep(0,ncol(x))
 
       #Margin 1
-      if(family$dist[1]=="normhnorm"){
+      if(family$family_mv[1]=="normhnorm"){
         m1<-mgcv::gam(formula=list(family$formula[[1]], family$formula[[2]], family$formula[[3]]),
                       data=family$data, family=normhnorm(s=family$s1), optimizer = c("efs"))
       }
-      if(family$dist[1]=="normexp"){
+      if(family$family_mv[1]=="normexp"){
         m1<-mgcv::gam(formula=list(family$formula[[1]], family$formula[[2]], family$formula[[3]]),
                       data=family$data, family=normexp(s=family$s1), optimizer = c("efs"))
       }
 
       #Margin 2
-      if(family$dist[2]=="normhnorm"){
+      if(family$family_mv[2]=="normhnorm"){
         m2<-mgcv::gam(formula=list(family$formula[[4]], family$formula[[5]], family$formula[[6]]),
                       data=family$data, family=normhnorm(s=family$s2), optimizer = c("efs"))
       }
-      if(family$dist[2]=="normexp"){
+      if(family$family_mv[2]=="normexp"){
         m2<-mgcv::gam(formula=list(family$formula[[4]], family$formula[[5]], family$formula[[6]]),
                       data=family$data, family=normexp(s=family$s2), optimizer = c("efs"))
       }
 
       start[-c(jj[[7]])]<-c(m1$coefficients,m2$coefficients)
 
-      u1<-pcomperr(q=y[,1], mu=m1$fitted.values[,1], sigma_v=m1$fitted.values[,2], par_u=m1$fitted.values[,3], s=family$s1, dist=family$dist[1], log.p=FALSE)
-      u2<-pcomperr(q=y[,2], mu=m2$fitted.values[,1], sigma_v=m2$fitted.values[,2], par_u=m2$fitted.values[,3], s=family$s2, dist=family$dist[2], log.p=FALSE)
+      u1<-pcomperr(q=y[,1], mu=m1$fitted.values[,1], sigma_v=m1$fitted.values[,2], par_u=m1$fitted.values[,3], s=family$s1, family=family$family_mv[1], log.p=FALSE)
+      u2<-pcomperr(q=y[,2], mu=m2$fitted.values[,1], sigma_v=m2$fitted.values[,2], par_u=m2$fitted.values[,3], s=family$s2, family=family$family_mv[2], log.p=FALSE)
 
       family$data$y3<-rep(1,nrow(Y))
       adj_par_formula<-update.formula(family$formula[[7]], y3 ~ .)
       m3<-mgcv::gam(list(adj_par_formula),
-                    data=family$data, family=cop(u1=u1, u2=u2, cop_family=family$dist[3]), optimizer = c("efs"))
+                    data=family$data, family=cop(W=as.matrix(cbind(u1,u2)), family_cop=family$family_mv[3]), optimizer = c("efs"))
 
       start[jj[[7]]] <- m3$coefficients
     }
   }) ## initialize
 
-  #Random number generation for normhnorm
+  #Random number generation for comperr_mv
   rd <- function(mu, wt, scale, family) {
     ## random number generation
     mu <- as.matrix(mu)
     if(ncol(mu)==2){ mu <- t(mu) }
 
-    return(rcomperr_mv(ncol(mu), mu1 = mu[,1], sigma_v1 = mu[,2], par_u1 = mu[,3], s1=family$s1, dist1=family$dist[1],
-                                 mu2 = mu[,4], sigma_v2 = mu[,5], par_u2 = mu[,6], s2=family$s2, dist2=family$dist[2],
-                                 Tau= mu[,7], family=family$dist[3]))
+    return(rcomperr_mv(ncol(mu), mu1 = mu[,1], sigma_v1 = mu[,2], par_u1 = mu[,3], s1=family$s1,
+                                 mu2 = mu[,4], sigma_v2 = mu[,5], par_u2 = mu[,6], s2=family$s2,
+                                 delta= mu[,7], family_mv=family$family_mv))
   } ## random number generation
 
-  #Cumulative distribution function of normhnorm
+  #Cumulative distribution function of comperr_mv
   cdf <- function(q, mu, wt, scale, logp, family) {
     ##cumulative distribution function
     mu <- as.matrix(mu)
     if(ncol(mu)==2){ mu <- t(mu) }
 
-    return(pcomperr_mv(q1=q[,1], mu1 = mu[,1], sigma_v1 = mu[,2], par_u1 = mu[,3], s1=family$s1, dist1=family$dist[1],
-                       q2=q[,2], mu2 = mu[,4], sigma_v2 = mu[,5], par_u2 = mu[,6], s2=family$s2, dist2=family$dist[2],
-                       Tau=mu[,7], family=family$dist[3],
-                       log.p = logp))
+    return(pcomperr_mv(q1=q[,1], mu1 = mu[,1], sigma_v1 = mu[,2], par_u1 = mu[,3], s1=family$s1,
+                       q2=q[,2], mu2 = mu[,4], sigma_v2 = mu[,5], par_u2 = mu[,6], s2=family$s2,
+                       delta=mu[,7], family_mv=family$family_mv),
+                       log.p = logp)
   } ##cumulative distribution function
 
   #Prediction function
@@ -351,8 +381,8 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
     }
 
 
-    para1<-reparametrize(mu = mu1, sigma_v = sigma_v1, par_u = par_u1, s=family$s1, dist=family$dist[1])
-    para2<-reparametrize(mu = mu2, sigma_v = sigma_v2, par_u = par_u2, s=family$s2, dist=family$dist[2])
+    para1<-reparametrize(mu = mu1, sigma_v = sigma_v1, par_u = par_u1, s=family$s1, family=family$family_mv[1])
+    para2<-reparametrize(mu = mu2, sigma_v = sigma_v2, par_u = par_u2, s=family$s2, family=family$family_mv[2])
 
 
     fv <- list(cbind(para1$mean,para2$mean))
@@ -376,7 +406,7 @@ comperr_mv <- function(link = list("identity", "log", "log", "identity", "log",
                  preinitialize=preinitialize,
                  s1 = s1,
                  s2 = s2,
-                 dist = dist,
+                 family_mv = family_mv,
                  residuals=residuals,
                  rd=rd,
                  cdf=cdf,
